function [k_ER, k_GR, ER, GR ] = AH( Xobs, kmax )
%% This function is used to calculate Ahn and Horenstein's (2013, Econometrica) Eigenvalue ratio test for the number of factors 
%% Input: Xobs: data, N x T 
%%        kmax: the upper bound of the number of factors 
%% Output: k_ER, k_GR: the optimal number of factors estimated by maxER 
%%         ER, GR: Ahn and Horenstein's (2013) estimators 

%% Written by Xia WANG, @SMU, Dec. 30, 2014

[T, N]=size(Xobs); 
XX = Xobs *Xobs'/(T*N);         % dim: T x T 
[Ffit0, D] = eig(XX); 

 muv = diag(D); 
 mu0 = flipud(muv); 
 
 ER = zeros(kmax,1); 
 GR = zeros(kmax,1); 
 
 %% calculate ER 
 ER = mu0(1:kmax)./mu0(2: kmax+1); 

 %% calculate GR 
 m=min(T, N); 
 V = zeros(kmax+2,1);     %V(1) corresponding to V(0) in the paper 
 for i = 1:kmax+2
     V(i) = sum(mu0(i:m));
 end 
 GR = log( V(1:kmax)./V(2:kmax+1))./log( V(2:kmax+1)./V(3:kmax+2)); 
 
 [~,k_ER] = max(ER); 
 [~,k_GR] = max(GR); 

end

