function [ Rp1, Rp2, Ri1, Ri2, PC1, PC2, IC1, IC2] = BN_num_factor( Xobs, Kmax)
%% This procedure calculates Bai and Ng's (2002) information criterion in selecting the number of factors in dynamic factor model
%% Input: Xobs: observed data 
%%        Kmax: number of factors 
%% Output: Bai and Ng's (2002) PC_p1, PC_p2, IC_p1, IC_p2 on page 201 and the selected optimal number of factors 


[T, N] = size(Xobs); 
XX = Xobs *Xobs'/(T*N);         % dim: T x T 
[Ffit0, D] = eig(XX); 
Fmax = Ffit0(:, T-Kmax+1:T)*sqrt(T); % dim: TxKmax, estimated factors by PCA
lamax =  Xobs' * Fmax/T;            % dim: NxKmax, estimated factor loadings
emax = Xobs- Fmax * lamax';        % dim: TxN, error term from time invariant factor models

sige2 = sum(sum(emax.^2,1))/(N*T); 
CNT = min(sqrt(T), sqrt(N));

PC1 = zeros(Kmax,1); 
PC2 = zeros(Kmax,1); 
IC1 = zeros(Kmax,1); 
IC2 = zeros(Kmax,1); 

for R = 1:Kmax
    F0 = Ffit0(:, T-R+1:T)*sqrt(T); % dim: TxR, estimated factors by PCA
    lam0 =  Xobs' * F0/T;            % dim: NxR, estimated factor loadings
    e = Xobs- F0 * lam0';        % dim: TxN, error term from time invariant factor models
    sig = sum(e.^2, 1)./T;          % dim: 1 x N
    V = N^(-1) * sum(sig);
    PC1(R) = V + R* sige2 * (N+T)/(N*T) * log(N*T/(N+T)); 
    PC2(R) = V + R* sige2 * (N+T)/(N*T) * log(CNT^2); 
    IC1(R) = log(V) + R * (N+T)/(N*T) * log(N*T/(N+T));
    IC2(R) = log(V) + R*(N+T)/(N*T) * log(CNT^2); 
end

[~,Rp1] =min(PC1); 
[~,Rp2] =min(PC2); 
[~,Ri1] =min(IC1); 
[~,Ri2] =min(IC2); 



end

