function [Xdata] = MultiNormal(mu, v, T)
%% This function is used to generate a multinormal distributed vector 
%% Written by Xia WANG, Jan. 5, 2015, @SMU, Singapore 
%% Input: mu: mean, dim: N x 1 
%%        v: variance, dim: N x N 
%% Output: Xdata: dim, T x N  
N = size(mu,1); 

LL = diag( diag(v).^(1/2)); 

[R, p] = chol(v);    % R is upper triangle, satisfies R'*R = A 
if p ~= 0
    disp('The variance matrix is not positive definite'); 
    sR = size(R,1); 
    LL(1:sR, 1:sR) = R; 
else 
    LL = R; 
end 

u = randn(N,T); 

x = mu*ones(1, T)  + LL * u; 
Xdata = x'; 




