function [ lam, sig ] = factor_loading_determine_factors( N, T,h,  DGP)
%% This function is used to generate factor loadings for the simulation studies of the paper 
%% On Time varying Factor Models: Estimation and Inference 
%% Written by Xia WANG, Jan. 5, 2015, @SMU, Singapore 
%% lam: N x r for DGP=1,2,3; and T x N x r for others
if DGP == 1 
    %% DGP.S1 in ET's paper 
   lam = randn(N,2);
   sig = 0;  
elseif DGP == 2
    %% DGP.S2 in ET's paper 
   lam = randn(N,2);
   sig = rand(N,1)+0.5; 
elseif DGP == 3
    %% DGP.S3 in ET's paper 
   lam = randn(N,2); 
   sig = rand(N,1)*0.2; 
elseif DGP == 4 
   %% DGP.S4 in ET's paper 
   %% time-invariant factor loadings + cross sectional dependence 
    lam = randn(N,2);
    sig = 0; 
elseif DGP == 5
    %% DGP.P1 in ET's paper 
     %% Single sudden structural break DGP.P1 at ET's paper 
   sig = rand(N,1)+0.5; 
   Tc = ceil(T/2); 
   b=0.2;
   lam = zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N);  
   lami = randn(N,1)+1;   
   lam1(1:Tc,:) = ones(Tc,1) *lami'; 
   lam1(Tc+1:T,:) = b + ones(T-Tc,1) * lami'; 
   lami2 = randn(N,1)+1;
   lam2(1:Tc,:) = ones(Tc,1) *lami2'; 
   lam2(Tc+1:T,:) = b + ones(T-Tc,1) * lami2';
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP == 6 
    %% DGP.P2 in ET's paper 
    %%  multiple structural changes-A nonpara. test for structural changes -DGP.P2
   sig = 0;
   Ti = (1:1:T)'; 
   T1c = ceil(0.1*T); 
   T2c = ceil(0.2*T); 
   T3c = ceil(0.4*T); 
   T4c = ceil(0.5*T); 
   T5c = ceil(0.7*T); 
   T6c = ceil(0.8*T); 
   lam = zeros(T,N,2); 
   lam1 = zeros(T,N); 
   lam2 = zeros(T,N); 
   lami = randn(N,1)+1; 
   lami2 = randn(N,1)+1; 
   b1 =0.5; 
   b2 = -0.5;  
   lam1 = ((Ti>T1c & Ti<T2c)|(Ti>T5c & Ti < T6c) )*(ones(1,N)*b1) + (Ti >T3c & Ti < T4c)*(ones(1,N)* b2) + ones(T,1)*lami'; 
   lam2 = ones(T,1)*lami2'; 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP == 7
     %% DGP.P3 in ET's paper 
    % Single smooth structural change
   sig = 0; 
   b=0.5; 
   lam= zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N); 
   Tc = (1:1:T)'/T;
   lam1 = ones(T,1)*randn(1, N); 
   uu = (1:1:N)/(2*N)+0.2; 
   mu = ones(T,1) * uu;
   %  lam1 = 1*(1+ exp(-(Tc*ones(1,N)- mu)/0.07)).^(-1);   
   lam2 = b*(1+ exp(-(Tc*ones(1,N)- mu)/0.05)).^(-1);
  % lam2 = b*(1+ exp(-(Tc*ones(1,N)- mu)/0.05)).^(-1)-0.5*b; 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP ==8 
     %% DGP.P4 in ET's paper 
    sig = 0; 
   Ti = (1:1:T)'/T; 
   T1 =  0.1; 
   T2 =  0.3; 
   T3 =  0.7; 
   T4 =  0.9; 
   b=0.5; 
   lam= zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N); 
   lami = randn(N,1)+1;    
   Gn = 1./(1+ exp(-1000 *(Ti-T1).*(Ti-T2).* (Ti-T3).* (Ti-T4)));    %dim: T x 1  
   lam1 = ones(T,1)*lami' +  Gn*b*ones(1,N); 
   lam2 = ones(T,1)* randn(1,N); 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP ==9 
    %% DGP.P5 in ET's paper 
    %% factor loadings with single structural break (DGP7) + cross sectional dependence 
  sig = rand(N,1)+0.5; 
   Tc = ceil(T/2); 
   b=0.2;    % b = 1,2, 4 
   lam = zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N);  
   lami = randn(N,1)+1;   
   lam1(1:Tc,:) = ones(Tc,1) *lami'; 
   lam1(Tc+1:T,:) = b + ones(T-Tc,1) * lami'; 
   lami2 = randn(N,1)+1;
   lam2(1:Tc,:) = ones(Tc,1) *lami2'; 
   lam2(Tc+1:T,:) = b + ones(T-Tc,1) * lami2';
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
   
elseif DGP ==10 
    %% DGP.P6 in ET's paper
    %%  multiple structural changes+cross sectional dependence -A nonpara. test for structural changes -DGP.P2
   sig = 0;
   Ti = (1:1:T)'; 
   T1c = ceil(0.1*T); 
   T2c = ceil(0.2*T); 
   T3c = ceil(0.4*T); 
   T4c = ceil(0.5*T); 
   T5c = ceil(0.7*T); 
   T6c = ceil(0.8*T); 
   lam = zeros(T,N,2); 
   lam1 = zeros(T,N); 
   lam2 = zeros(T,N); 
   lami = randn(N,1)+1; 
   lami2 = randn(N,1)+1; 
   b1 =0.5; 
   b2 = -0.5;  
   lam1 = ((Ti>T1c & Ti<T2c)|(Ti>T5c & Ti < T6c) )*(ones(1,N)*b1) + (Ti >T3c & Ti < T4c)*(ones(1,N)* b2) + ones(T,1)*lami'; 
   lam2 = ones(T,1)*lami2'; 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP ==11 
    %% DGP.P7 in ET's paper
   %% factor loadings with smooth structural changes + cross sectional dependence 
   sig = 0; 
   c = 0.5;              % c = 1,2, 4 
   lam= zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N); 
   Tc = (1:1:T)'/T;
   lam1 = ones(T,1)*randn(1, N); 
   uu = (1:1:N)/(2*N)+0.2; 
   mu = ones(T,1) * uu;
   lam2 = c*(1+ exp(-(Tc*ones(1,N)- mu)/0.05)).^(-1); 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP == 12 
    %% DGP.P8 in ET's paper heterogenous time variations 
       sig = 0;
   Ti = (1:1:T)'; 
   T1c = ceil(0.1*T); 
   T2c = ceil(0.2*T); 
   T3c = ceil(0.4*T); 
   T4c = ceil(0.5*T); 
   T5c = ceil(0.7*T); 
   T6c = ceil(0.8*T); 
   lam = zeros(T,N,2); 
   lam1 = zeros(T,N); 
   lam2 = zeros(T,N); 
   lami = randn(N,1)+1; 
   lami2 = randn(N,1)+1; 
   nui = rand(N,1); 
   lam1(:, 1:N) = ((Ti>T1c & Ti<T2c)|(Ti>T5c & Ti < T6c) )*nui' + (Ti >T3c & Ti < T4c)* (-nui)' + ones(T,1)*lami(1:N)'; 
   lam2 = ones(T,1)*lami2'; 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2; 
elseif DGP == 13 
    %% DGP.P9, local power 1
    c = 1;              % c =  1, 2, 4 
    aNT = T^(-1/2)*N^(-1/4)*h^(-1/4);  % convergence rate of our test 
   sig = rand(N,1)+0.5; 
   Tc = ceil(T/2); 
   lam = zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N);  
   lami1 = randn(N,1)+1;
   git1 = zeros(T,N); 
   git1(Tc+1:T, :) =  ones(T-Tc,N); 
   lam1 = ones(T,1)*lami1' + aNT * c * git1; 
   
   lami2 = randn(N,1)+1;
   git2 = zeros(T,N); 
   git2(Tc+1:T, :) =  ones(T-Tc, 1)*ones(1,N); 
   lam2 = ones(T,1)*lami2' + aNT * c * git2; 
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
elseif DGP == 14; 
    %% DGP.P10 local power 2: g_i(t/T) same as DGP 6
   c = 1;                             % c = 1, 2, 4 
   aNT = T^(-1/2)*N^(-1/4)*h^(-1/4);  % convergence rate of our test 
   sig = 0; 
   lam= zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N); 
   Tc = (1:1:T)'/T;
   lam1 = ones(T,1)*randn(1, N); 
   mu = ones(T,N) * 0.5;
   git = (1+ exp(-(Tc*ones(1,N)- mu)/0.05)).^(-1); 
   lam2 = ones(T,1)*randn(1, N) + aNT * c * git;
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2; 
elseif DGP == 15 
    %% DGP.P11 
    sig = rand(N,1)+0.5;
   c =0.5;   % c = 0.5, 0.6, 0.7, 0.8, 0.9 
   Tc = ceil(c*T); 
   b=0.2;    % b = 0.2, 0.5 
   lam = zeros(T,N,2); 
   lam1 = zeros(T, N); 
   lam2 = zeros(T, N);  
   lami = randn(N,1)+1;   
   lam1(1:Tc,:) = ones(Tc,1) *lami'; 
   lam1(Tc+1:T,:) = b + ones(T-Tc,1) * lami'; 
   lami2 = randn(N,1)+1;
   lam2(1:Tc,:) = ones(Tc,1) *lami2'; 
   lam2(Tc+1:T,:) = b + ones(T-Tc,1) * lami2';
   lam(:,:,1) = lam1; 
   lam(:,:,2) = lam2;
end 


end

