clear 
clc 
 %% ****************************************************% 
 %% This is the main function for the paper
 %% Testing for Structural Changes in Factor Models via a Nonparametric Regression  
 %% Written by Xia WANG, Aug. 7, 2018, @SYU, China 
 %% get the result of simulation with bootstrap 
 %% determine the number of common factors by Bai and Ng's (2002) IC
 %% ****************************************************

N =100; 
T =100; 
DGP = 1; 
Rmax = 6;                       % upper bound of the #factors 
rep = 10; 
BB = 200;  

h2 =  2.35 * 1/sqrt(12)  * T^(-1/5); 
kern = 2;                       % Epanechnikov kernel

SM = zeros(rep, 1); 
SM1 = zeros(rep, 1); 
SM2 = zeros(rep, 1); 


QV_5=ones(rep,1)*10; 
QV_10=ones(rep,1)*10; 
CTb5=ones(rep,1)*10; 
CTb10=ones(rep,1)*10; 


QV1_5=ones(rep,1)*10; 
QV1_10=ones(rep,1)*10; 
CTb15=ones(rep,1)*10; 
CTb110=ones(rep,1)*10; 

QV2_5=ones(rep,1)*10; 
QV2_10=ones(rep,1)*10; 
CTb25=ones(rep,1)*10; 
CTb210=ones(rep,1)*10; 

 

   randn('seed',20180910 );
   rand('seed',20180910 );

% [ lam, sig ] = factor_loading( N, T, h, DGP); 
[ lamg, sig ] = factor_loading_determine_factors( N, T, h2, DGP);
lam = lamg .*(lamg < 2*ones(size(lamg))) + 2* (lamg >2*ones(size(lamg))); 


 
 %% loop start 
parfor r = 1: rep
    r
    randn('seed',r+1);
    rand('seed',r+1);
    SMb = zeros(BB, 1);
    SMb1 = zeros(BB, 1);
    SMb2 = zeros(BB, 1);
    
    %****************************************************************
    % 1. Data generation and normalization for Each DGP
    % ***************************************************************
    %% random factor loadings
    [ lamg, sig ] = factor_loading_determine_factors( N, T, h2, DGP);
    
    lam = lamg .*(lamg < 2*ones(size(lamg))) + 2* (lamg >2*ones(size(lamg)));
    
    [ data, F, lamtrue ] = DGP_determine_factors(N, T, h2, DGP, lam, sig );
    % data normalization
    m = mean(data)';         % dim: N x 1
    s = std(data)';          % dim: N x 1
    u = ones(T,1);
    data = (data - u*m')./(u*s');
    X = data;
    
    %% Bai and Ng's (2002) IC 
    [ ~, ~, r1, r2, ~, ~, ~, ~] = BN_num_factor( X, Rmax); 
     r2 = 2; 
    
    [lam02, ~, F02, MM ] = mod_test_tvloading(X , r2, h2 , kern);
    
    [MM1, MM2,~,~,~,~] = test_common_component(X, r2, h2 , kern);
     
     
    SM(r) = MM;
     SM1(r) = MM1; 
     SM2(r) = MM2; 
    
    %****************************************************************
    % 2. Start bootstrap: use wild bootstrap
    % ***************************************************************
    
    %************************************
     ev2 = X - F02*lam02';          % dim:  T x N             
    ev2 = ev2 - u* mean(ev2);
    N0 = ceil(N^(1/3));
    A1 = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]);
    Nc = (A1<=N0);
    mu = zeros(N,1);
    
    %    Ce = (cov(ev)+  0.01/(N*T)*eye(N)).*Nc;
    Ce2 = (cov(ev2)+  0.01/(N*T)*eye(N)); 
    
    for b = 1: BB
       
        estar = MultiNormal(mu, Ce2, T);
        Xstar = F02*lam02' + estar;
        m = mean(Xstar)';
        s = std(Xstar)';
        u = ones(T,1);
        Xstar = (Xstar - u*m')./(u*s');
       
       [~,~,~, SMb(b)] = mod_test_tvloading(Xstar , r2, h2 , kern);
       [SMb1(b), SMb2(b),~,~,~,~] = test_common_component(Xstar, r2, h2 , kern);
       
    end
    %% SM
        
        
        QV_5(r) = quantile(SMb , 0.95);
        QV_10(r) = quantile(SMb, 0.90);
    
        [SM(r) QV_5(r) QV_10(r)]
    
        CTb5(r) = (SM(r)>QV_5(r));
        CTb10(r) = (SM(r)>QV_10(r));
        
        
            %% SM1 AND SM2 
        QV1_5(r) = quantile(SMb1 , 0.95);
        QV1_10(r) = quantile(SMb1, 0.90);
    
        [SM1(r) QV1_5(r) QV1_10(r)]
    
        CTb15(r) = (SM1(r)>QV1_5(r));
        CTb110(r) = (SM1(r)>QV1_10(r));
        
        QV2_5(r) = quantile(SMb2 , 0.95);
        QV2_10(r) = quantile(SMb2, 0.90);
    
        [SM2(r) QV2_5(r) QV2_10(r)]
    
        CTb25(r) = (SM2(r)>QV2_5(r));
        CTb210(r) = (SM2(r)>QV2_10(r));
%         
        
end
  
  ss=mean([CTb5 CTb10 CTb15 CTb110 CTb25 CTb210]) 
   
  
  % csvwrite('N100T100DGP1ss.csv', ss);
 
