%****************************************************
% This is the main function for the paper
% Testing for structural changes in factor models via a nonparametric
% regression 
% Written by Xia WANG, Dec. 17, 2014, @SMU, Singapore
% get the result of simulation with bootstrap 
% Written by Xia WANG 
% Sep. 2014 @ Singapore 
%****************************************************
clear 
clc
N = 100; 
T = 100; 
DGP = 1; 
 

h = 2.35 * 1/sqrt(12)  * T^(-1/5); 

kern = 2;                       % Epanechnikov kernel
R = 2;                          % #common factors 

BB = 200;                        % #bootstrap iteration 
rep = 500;                       % # replication 

SM = zeros(rep, 1); 
SM1 = zeros(rep, 1); 
SM2 = zeros(rep, 1); 

QV_5=ones(rep,1)*10; 
QV_10=ones(rep,1)*10; 
CTb5=ones(rep,1)*10; 
CTb10=ones(rep,1)*10;

QV1_5=ones(rep,1)*10; 
QV1_10=ones(rep,1)*10; 
CTb15=ones(rep,1)*10; 
CTb110=ones(rep,1)*10;

QV2_5=ones(rep,1)*10; 
QV2_10=ones(rep,1)*10; 
CTb25=ones(rep,1)*10; 
CTb210=ones(rep,1)*10;

%% loop starts 
% [ lam, sig ] = factor_loading( N, T, h, DGP)
%      randn('seed',123456);
%      rand('seed',123456);
      randn('seed',123456789);
      rand('seed',123456789);

%% nonrandom factor loadings        
 % [ lamg, sig ] = factor_loading_determine_factors( N, T, h2, DGP);
% lam = lamg .*(lamg < 2*ones(size(lamg))) + 2* (lamg >2*ones(size(lamg))); 


   parfor r = 1: rep
    r
    randn('seed',r+1);
    rand('seed',r+1);
    SMb = zeros(BB, 1);
    SMb1 = zeros(BB, 1);
    SMb2 = zeros(BB, 1);
    %****************************************************************
    % 1. Data generation and normalization for Each DGP 
    % ***************************************************************
 %% random factor loadings   
    [ lamg, sig ] = factor_loading_determine_factors( N, T, h, DGP); 
    lam = lamg .*(lamg < 2*ones(size(lamg))) + 2* (lamg >2*ones(size(lamg))); 
   [ data, F, lamtrue ] = DGP_determine_factors(N, T, h, DGP, lam, sig );
    
   % data normalization 
    m = mean(data)';         % dim: N x 1 
    s = std(data)';          % dim: N x 1 
    u = ones(T,1); 
    data = (data - u*m')./(u*s');
    X = data; 
    
    %% ET's test 
    [lam0, ~, F0, MM ] = mod_test_tvloading(X , R, h , kern);
    SM(r) = MM;
    
    %% Su and Wang (2017, JoE) test 
    [M1, M2,~,~, lam0, F0] = test_common_component(X , R, h , kern);
    SM1(r) = M1;
    SM2(r) = M2; 
    
    %****************************************************************
    % 2. Start bootstrap: use wild bootstrap  
    % ***************************************************************
    ev = X - F0*lam0';          % dim:  T x N
    
    sde= std(ev);                % dim: 1 x N 
    ev = ev - u* mean(ev); 
    N0 = ceil(N^(1/3)); 
    A1 = abs([1:1:N]'*ones(1,N) - ones(N,1)*[1:1:N]); 
    Nc = (A1<=N0); 
    mu = zeros(N,1);
        

 Ce = (cov(ev)+  0.01/(N*T)*eye(N)); 
%     [RR, p] = chol(Ce);    % R is upper triangle, satisfies R'*R = A 
%     ppp=p
%     size(RR)
%     if p ~= 0
%         disp('The variance matrix is not positive definite');
%         
%     end

    
    for b = 1: BB  
        estar = MultiNormal(mu, Ce, T);
        Xstar = F0*lam0' + estar;
        m = mean(Xstar)';
        s = std(Xstar)';
        u = ones(T,1);
        Xstar = (Xstar - u*m')./(u*s');
       [~,~,~, SMb(b)] = mod_test_tvloading(Xstar , R, h , kern);
        
       [SMb1(b), SMb2(b),~,~,~,~] = test_common_component(Xstar, R, h , kern); 
      
    end 
    %%% SM 
      QV_5(r) = quantile(SMb , 0.95);
     QV_10(r) = quantile(SMb, 0.90);
     
  %   [SM(r) QV_5(r) QV_10(r)] 
    
    CTb5(r) = (SM(r)>QV_5(r));
    CTb10(r) = (SM(r)>QV_10(r));
    
    %%% SM1
    QV1_5(r) = quantile(SMb1 , 0.95);
    QV1_10(r) = quantile(SMb1, 0.90);
    
    [SM1(r) QV1_5(r) QV1_10(r)] 
    
    CTb15(r) = (SM1(r)>QV1_5(r));
    CTb110(r) = (SM1(r)>QV1_10(r));
     
    %%% SM2 
    QV2_5(r) = quantile(SMb2 , 0.95);
    QV2_10(r) = quantile(SMb2, 0.90);
    
    [SM2(r) QV2_5(r) QV2_10(r)] 
    
    CTb25(r) = (SM2(r)>QV2_5(r));
    CTb210(r) = (SM2(r)>QV2_10(r));
    
 end
%     meanSQ= mean(SM)
%     stdSQ=  std(SM) 
%      [ SM  QV_5  QV_10 CTb5 CTb10 ]  
%     
      criv90=icdf('norm', 0.9 , 0, 1);
      criv95=icdf('norm', 0.95 , 0, 1);
      criv99=icdf('norm', 0.99 , 0, 1);
    
    %% SM: ET's Test  
    zz90=(SM > criv90 *ones(rep,1));
    zz95=(SM >criv95 *ones(rep,1));
    zz99=(SM >criv99 *ones(rep,1));
    size90=sum(zz90)/rep
    size95=sum(zz95)/rep
    size99=sum(zz99)/rep   
    ssET=mean([CTb5 CTb10])  
    
    %%% SM1 & SM2: JoE's test On Time Varying Factor Models 
    [ SM1  QV1_5  QV1_10 CTb15 CTb110 ]
    
    zz90=(SM1 > criv90 *ones(rep,1));
    zz95=(SM1 >criv95 *ones(rep,1));
    zz99=(SM1 >criv99 *ones(rep,1));
    size90=sum(zz90)/rep
    size95=sum(zz95)/rep
    size99=sum(zz99)/rep   
    ssJOE=mean([CTb15 CTb110])    
    
    
    [ SM2  QV2_5  QV2_10 CTb25 CTb210 ]
    zz90=(SM2 > criv90 *ones(rep,1));
    zz95=(SM2 >criv95 *ones(rep,1));
    zz99=(SM2 >criv99 *ones(rep,1));
    size90=sum(zz90)/rep
    size95=sum(zz95)/rep
    size99=sum(zz99)/rep   
    ssJOE2=mean([CTb25 CTb210])       
    