clear
clc

%****************************************************
% This is the main function for the paper
% Testing for Structural Changes in Factor Models via a Nonparametric Regression  
% Written by Xia WANG, Aug. 7, 2018, @SYU, China 
% get the result of simulation without bootstrap 
 
%****************************************************
tic
N =100 ; 
T =100 ; 
DGP =1; 
 % h =  2.35/sqrt(12) *  T^(-1/5)*N^(-1/10); 
 h =  2.35/sqrt(12) *  T^(-1/5) ; 
 
kern = 2;                     % Epan. kernel function 
R = 2; 

tau = 0.15; 

Kmax = 6;    % upper bound of factors in BN criterion
 
rep =500  ;  % number of repetitions 
SM = ones(rep, 1);  % for statistics
SM1 = ones(rep, 1);  % for statistics
SM2 = ones(rep, 1);  % for statistics
SM3 = ones(rep, 1);  % for statistics
SM4 = ones(rep, 1);  % for statistics



HI_LM = ones(rep, 1);  % for statistics
HI_W = ones(rep, 1);  % for statistics
HI_crit = ones(rep, 2); 


CDG_LM = ones(rep, 1);  % for statistics
CDG_W = ones(rep, 1);  % for statistics
CDG_crit = ones(rep, 2); 

 
BE_LM = ones(rep, N);  
BE_crit = ones(rep, 2); 
randn('seed', 20180910 );
rand('seed', 20180910 );


% [ lam, sig ] = factor_loading( N, T, h, DGP); 
[ lam, sig ] = factor_loading_determine_factors( N, T, h, DGP);    % generate data for factor loading and error term 

lamT0 = zeros(N, rep);
lamTT = zeros(T, N, rep);
F0T = zeros(T, rep);
F2T = zeros(T, rep);
RP1 = zeros(rep,1); 
RP2 = zeros(rep,1); 
RI1 = zeros(rep,1); 
RI2 = zeros(rep,1); 
BE_LMN5 = zeros(rep,1);
BE_LMN10  = zeros(rep,1);

% generate data X_{it} 
 [ data, F, lamtrue ] = DGP_determine_factors(N, T, h, DGP, lam, sig );

m = mean(data)';         % dim: N x 1
s = std(data)';          % dim: N x 1
u = ones(T,1);
data = (data - u*m')./(u*s');
X = data;
[~, ~, HIcrit] = HI_test(X, R, tau);
[supLM, supW, CDGcrit] = CDG_test(X, R, tau);
[supLM, BEcrit] = BE_test(X, R, tau);


%% loop start 
  parfor r = 1: rep
    r
     randn('seed',r+10086 );
      rand('seed',r+10086 );
    %****************************************************************
    % 1. Data generation and normalization for Each DGP
    % ***************************************************************
    % [ data, F, lamtrue ] = DGP_test_tvloading(N, T, h, DGP, lam, sig );
    [ data, F, lamtrue ] = DGP_determine_factors(N, T, h, DGP, lam, sig );
    
    m = mean(data)';         % dim: N x 1
    s = std(data)';          % dim: N x 1
    u = ones(T,1);
    data = (data - u*m')./(u*s');
    X = data; 
    
%     %% determine the optimal number of common factors 
    [ Rp1, Rp2, Ri1, Ri2, ~, ~, ~, ~] = BN_num_factor( X, Kmax); 
    RP1(r) = Rp1; 
    RP2(r) = Rp2; 
    RI1(r) = Ri1; 
    RI2(r) = Ri2;
    R = Ri2; 
    %% or fixed the number of common factors 
     R= 2; 
    
    %% Han and Inoue's (2014, ET) test
    [supLM, supW, crit] = HI_test(X, R, tau); 
    HI_LM(r) = supLM; 
    HI_W(r) = supW; 
    HI_crit(r, :) = crit; 
  %  HI_crit = crit;  %[5% critical value, 10% critical value]
    
    %% Chen, Dolado and Gonzalo's (2014, JOE) test 
    [supLM, supW, crit] = CDG_test(X, R, tau);
    CDG_LM(r) = supLM; 
    CDG_W(r) = supW; 
    CDG_crit(r, :)= crit; 
 %   CDG_crit = crit;  %[5% critical value, 10% critical value]
   
    %% Breitung and Eickmeier's (2011, JOE) test 
    [supLM, crit] = BE_test(X, R, tau);   % remark: supLM: dim: 1 x N 
    BE_LM(r,:) = supLM;  
    BE_crit(r, :) = crit; 
   % BE_crit = crit;
   % lamT0(:, r) = lam0;
   % F0T(:, r) = F0;
   
   %follow one referee's suggetion to reject the null if more than 5 (10)
   % percent of the individual tests reject
%    BE_LMN5(r,1) = (sum((supLM> crit(:,1) *ones(1,N)))>0.05*N); 
%    BE_LMN10(r,1) = (sum((supLM> crit(:,2) *ones(1,N)))>0.1*N); 

  end 
    
    HILMz90=(HI_LM >HI_crit(:,2));
    HILMz95=(HI_LM >HI_crit(:,1));
    HIWz90=(HI_W >HI_crit(:,2));
    HIWz95=(HI_W >HI_crit(:,1));
    
     
    HILMsize90=sum(HILMz90)/rep; 
    HILMsize95=sum(HILMz95)/rep; 
    HIWsize90=sum(HIWz90)/rep; 
    HIWsize95=sum(HIWz95)/rep; 
    
    CDGLMz90=(CDG_LM > CDG_crit(:,2));
    CDGLMz95=(CDG_LM > CDG_crit(:,1));
    CDGWz90=(CDG_W > CDG_crit(:,2));
    CDGWz95=(CDG_W > CDG_crit(:,1));
    
     
    CDGLMsize90=sum(CDGLMz90)/rep; 
    CDGLMsize95=sum(CDGLMz95)/rep; 
    CDGWsize90=sum(CDGWz90)/rep; 
    CDGWsize95=sum(CDGWz95)/rep; 
    
    BELMz90=(BE_LM > BE_crit(:,2) *ones(1,N));
    BELMz95=(BE_LM > BE_crit(:,1) *ones(1,N));  
     
    BELMsize90=sum(sum(BELMz90)')/(rep*N); 
    BELMsize95=sum(sum(BELMz95)')/(rep*N);  

      
  % SMsize = [ SMsize95 SMsize90 ]*100 
    
   
   HI_LMsize = [ HILMsize95 HILMsize90]*100  
   HI_Wsize = [HIWsize95 HIWsize90 ]*100 
   CDG_LMsize = [CDGLMsize95 CDGLMsize90 ]*100  
   CDG_Wsize = [CDGWsize95 CDGWsize90 ]*100 
   BE_Wsize = [BELMsize95 BELMsize90 ]*100
   
  [  HI_LMsize HI_Wsize CDG_LMsize CDG_Wsize BE_Wsize ] 
  
 